package de.intevation.flys.exports;

import org.jfree.data.Range;
import org.jfree.data.xy.XYDataset;

import org.apache.log4j.Logger;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class ChartHelper {

    private static final Logger logger = Logger.getLogger(ChartHelper.class);


    /**
     * This method returns the ranges of the XYDataset <i>dataset</i> as array
     * with [xrange, yrange].
     *
     * @param dataset The dataset which should be evaluated.
     *
     * @return an array with x and y ranges.
     */
    public static Range[] getRanges(XYDataset dataset) {
        int seriesCount = dataset != null ? dataset.getSeriesCount() : 0;

        if (seriesCount == 0) {
            logger.warn("Dataset is empty or has no Series set.");
            return null;
        }

        boolean foundValue = false;

        double minX = Double.MAX_VALUE;
        double maxX = -Double.MAX_VALUE;
        double minY = Double.MAX_VALUE;
        double maxY = -Double.MAX_VALUE;

        for (int i = 0, m = seriesCount; i < m; i++) {
            for (int j = 0, n = dataset.getItemCount(i); j < n; j++) {
                double x = dataset.getXValue(i, j);
                double y = dataset.getYValue(i, j);

                if (Double.isNaN(x) || Double.isNaN(y)) {
                    logger.warn("Item " + j + " in Series " + i + " is broken");
                    continue;
                }

                foundValue = true;

                if (x < minX) {
                    minX = x;
                }

                if (x > maxX) {
                    maxX = x;
                }

                if (y < minY) {
                    minY = y;
                }

                if (y > maxY) {
                    maxY = y;
                }
            }
        }

        return foundValue
            ? new Range[] { new Range(minX, maxX), new Range(minY, maxY) }
            : null;
    }


    /**
     * Expand range by percent.
     *
     * @param range The range to expand.
     * @param percent The percentage to expand.
     *
     * @param an expanded range.
     */
    public static Range expandRange(Range range, double percent) {
        if (range == null) {
            return null;
        }

        double value  = range.getLowerBound();
        double expand = Math.abs(value / 100 * percent);

        return expand != 0
            ? new Range(value-expand, value+expand)
            : new Range(-0.01 * percent, 0.01 * percent);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
